package experiment.event.capability;

import experiment.property.Position;
import experiment.entity.Node;
import experiment.ExperimentState;
import visual.entity.Node3d;
import walker.Weyes;
import walker.exceptions.badSyntaxException;

/**
 * Created by IntelliJ IDEA.
 * User: andy
 * Date: Mar 11, 2010
 *
 * Includes the update information for a single Node
 */
public class NodeValueEvent extends ValueEvent {

    private String nodeId;
    private Position newPosition;

    /**
     * Default Constructor.
     *
     * @param timestamp the exact experiment moment that this event occured.
     * @param nodeId the node's Id
     * @param newPosition the Node's new position
     */
    public NodeValueEvent(double timestamp, String nodeId, Position newPosition) {
        this.timestamp = timestamp;
        this.nodeId = nodeId;
        this.newPosition = newPosition;
    }

    /**
     * Simple Constructor.
     *
     * @param timestamp the exact experiment moment that this event occured.
     * @param id the node's Id
     */
    public NodeValueEvent(long timestamp, String id) {
        this.timestamp = timestamp;
        this.nodeId = id;
    }


    /**
     * Applies this event to the experiment.
     *
     * @param state the experiment state where this event belong to.
     * @return true if event was successfully applied.
     * @throws walker.exceptions.badSyntaxException in case of a mistake in the wiseml syntax.
     */
    public boolean apply(ExperimentState state) throws badSyntaxException {

        // check that this is a valid NodeValueEvent.
        if (nodeId == null){
            return false;
        }

        // Get the instance of the Node3d.
        Node n = state.indexNode(nodeId);

        // If the Node doesn't exist add it to the experiment setup.
        if (n == null){
            n = new Node(state, nodeId, null, null, null, null, null);
            state.add(n);
        }

        // Update the corresponding state's timestamp.
        state.setTime(this.timestamp);

        // Update its seeked.
        if (newPosition != null){
            n.updatePosition(newPosition);
        }

        // Loop through the directiveList and update all known capabilities.
        for (Directive aDirective : directiveList) {
            n.setCapability(aDirective.getKey(), aDirective.getData());
                        
            if(Weyes.NodeDisplayCapability != null 
            		&& aDirective.getKey() != null
            		&& aDirective.getKey().equals(Weyes.NodeDisplayCapability)
            		&& aDirective.getData() != null) {
            	((Node3d)n.getEntity3d()).changeColor(aDirective.getData());
            }
            
            if(Weyes.NodeInfoCapability != null
            		&& aDirective.getKey() != null
            		&& aDirective.getKey().equals(Weyes.NodeInfoCapability)
            		&& aDirective.getData() != null) {
            	((Node3d)n.getEntity3d()).changeInfoText(aDirective.getKey(), aDirective.getData());
            }
        }

        return true;
    }
}
