package experiment.event.topology;

import experiment.event.Event;
import experiment.event.capability.Directive;
import experiment.ExperimentState;
import experiment.entity.Node;
import walker.exceptions.badSyntaxException;

/**
 * Created by IntelliJ IDEA.
 * User: andy
 * Date: Mar 11, 2010
 */
public class NodeEvent extends Event {

     private boolean status;
     private String nodeId;

    /**
     * Default constructor for Node Topology events.
     *
     * @param timestamp the Event timestamp
     * @param id the id of the node that this event corresponds to
     * @param status true to enable the entity, false to disable it
     */
    public NodeEvent(long timestamp, String id, boolean status) {
        this.timestamp = timestamp;
        this.nodeId = id;
        this.status = status;
    }

    /**
     * Applies this event to the experiment.
     *
     * @param state the experiment state where this event belong to.
     * @return true if event was successfully applied. 
     */
    public boolean apply(ExperimentState state) throws badSyntaxException {

        // check that this is a valid NodeEvent.
        if (nodeId == null){
            return false;
        }

        // Try to index the Node3d.
        Node node = state.indexNode(nodeId);

        // If the Node doesn't exist add it to the experiment setup.
        if (node == null){
            node = new Node(state, nodeId, null, null, null, null, null);
            state.add(node);
        }

        // If there has been a change
        if (status!=node.isEnabled()){
            // update the node's status.
            node.setStatus(status);
            // update the Node3d status
            node.getEntity3d().updateStatus();
        }

        // update the state's timestamp.
        state.setTime(timestamp);
        return true;
    }

    /**
     * Dummy method
     * @param data the data to be added
     */
    public void addData(Directive data) {
    }
}
