package visual;

/*
 *      Axis3d.java 1.0 98/11/25
 *
 * Copyright (c) 1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Sun grants you ("Licensee") a non-exclusive, royalty free, license to use,
 * modify and redistribute this software in source and binary code form,
 * provided that i) this copyright notice and license appear on all copies of
 * the software; and ii) Licensee does not utilize the software in a manner
 * which is disparaging to Sun.
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. SUN AND ITS LICENSORS SHALL NOT BE
 * LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
 * OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL SUN OR ITS
 * LICENSORS BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
 * INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
 * CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF
 * OR INABILITY TO USE SOFTWARE, EVEN IF SUN HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 *
 */

import javax.media.j3d.*;
import javax.vecmath.Point3f;

/**
 * Created by IntelliJ IDEA.
 * User: andy
 * Date: Apr 6, 2010
 *
 * A Shape3d object that contains all functionality and data for creating x y z axis in a 3d cancas.
 * The axis are of length 1.0.
 */
public class Axis3d extends Shape3D{

    public Axis3d() {

        this.setGeometry(createGeometry());
        this.setAppearance(createAppearance());

    }

    /**
     * Define the appearance of the axis lines.
     *
     * @return an Appearance object.
     */
    private Appearance createAppearance() {

        Appearance app = new Appearance();

        LineAttributes line = new LineAttributes();
        line.setLineAntialiasingEnable(true);
        line.setLineWidth(1f);
        app.setLineAttributes(line);

        ColoringAttributes col = new ColoringAttributes(Palette.red, ColoringAttributes.SHADE_FLAT);
        app.setColoringAttributes(col);
        
        return app;
    }

     /**
     * Creates the geometry that defines the 3 axis.
     *
     * @return a Geometry object.
     */
    private Geometry createGeometry(){

        // create an indexed multi-line that 'is' the 3 axis.
        IndexedLineArray axisLines = new IndexedLineArray(12, GeometryArray.COORDINATES | GeometryArray.COLOR_3, 18);

        axisLines.setCoordinate( 0, new Point3f( 0.0f, 0.0f, 0.0f));
        axisLines.setCoordinate( 1, new Point3f(-1.0f, 0.0f, 0.0f));
        axisLines.setCoordinate( 2, new Point3f( 1.0f, 0.0f, 0.0f));
        axisLines.setCoordinate( 7, new Point3f( 0.98f, 0.02f, 0.0f));
        axisLines.setCoordinate( 8, new Point3f( 0.98f, -0.02f, 0.0f));

        axisLines.setCoordinate( 3, new Point3f( 0.0f,-1.0f, 0.0f));
        axisLines.setCoordinate( 4, new Point3f( 0.0f, 1.0f, 0.0f));
        axisLines.setCoordinate( 9, new Point3f( 0.02f, 0.98f, 0.0f));
        axisLines.setCoordinate(10, new Point3f( -0.02f, 0.98f, 0.0f));

        axisLines.setCoordinate( 5, new Point3f( 0.0f, 0.0f,-1.0f));
        axisLines.setCoordinate( 6, new Point3f( 0.0f, 0.0f, 1.0f));


        axisLines.setCoordinateIndex( 0, 0);
        axisLines.setCoordinateIndex( 1, 1);

        axisLines.setCoordinateIndex( 2, 2);
        axisLines.setCoordinateIndex( 3, 7);
        axisLines.setCoordinateIndex( 4, 2);
        axisLines.setCoordinateIndex( 5, 8);
        axisLines.setCoordinateIndex( 6, 2);

        axisLines.setCoordinateIndex( 7, 0);
        axisLines.setCoordinateIndex( 8, 3);
        axisLines.setCoordinateIndex( 9, 4);
        axisLines.setCoordinateIndex( 10, 9);
        axisLines.setCoordinateIndex( 11, 4);
        axisLines.setCoordinateIndex( 12, 10);
        axisLines.setCoordinateIndex( 13, 4);

        axisLines.setCoordinateIndex( 14, 0);
        axisLines.setCoordinateIndex( 15, 5);
        axisLines.setCoordinateIndex( 16, 6);

        return axisLines;

    }


}
