package visual;

import experiment.property.Capability;
import experiment.entity.Entity;
import experiment.entity.Node;
import experiment.entity.Link;
import experiment.Setup;

import javax.swing.*;
import javax.vecmath.Vector3d;
import java.util.Date;
import java.text.SimpleDateFormat;

import visual.entity.Entity3d;

/**
 * Created by IntelliJ IDEA.
 * User: andy
 * Date: Apr 7, 2010
 */
public class FrontEnd extends Thread{

    private JPanel container;

    private JTextArea nodeId;
    private JTextArea position;
    private JTextArea capabilities;
    private JLabel time;

    public Entity entity = null;
    private boolean entityChanged = false;
    public final Object lock = new Object();

    private SimpleDateFormat format = new SimpleDateFormat("EEE MMM dd HH:mm:ss:SSS");

    public void run(){
//        container.setMinimumSize(new Dimension(170, Rendering.getScreenHeight()/5));

        // Wait until the user choses an entity to monitor.
        while (entity == null){
            synchronized (lock){
                try {
                    lock.wait();
                } catch (InterruptedException e) {
                    e.printStackTrace();
                }
            }
        }

        // Loop continuously
        while (true){

            // Update the gui.
            update();

            // Await a new change. The thread wakes up in two cases.
            // First if the monitored entity changes and secondly if the monitored entity is updated.
            synchronized (lock){
                try {
                    lock.wait();
                } catch (InterruptedException e) {
                    e.printStackTrace();
                }
            }
        }

    }

    /**
     * Is called by Engine3d, which extends frame.
     *
     * @return the JPanel that server as the top container for this GUI.
     */
    public JPanel getContainer() {
        return container;
    }

    /**
     * When a new picking action is caught, if the user has chosen to change the monitored entity,
     * engine3d calls this method to notify the FrontEnd thread for the change. In the case that the previous
     * monitored Entity doesn't exist any more, this happens if the a seek to an older state  that the monitoredEntity
     * didn't exist, the method is called with a null parameter so as to avoid shrinking the invisible but existing
     * Entity3d.
     *
     * @param entity the entity that the user wants to monitore.
     */
    public synchronized void changeViewedEntity(Entity entity){

        entityChanged = true;

        // if the new monitored Entity exists,
        if (entity != null) {

            // Notify the previous monitored entity that it is not being monitored any more.
            if (this.entity != null){
                this.entity.setMonitored(false);
                Entity3d old = this.entity.getEntity3d();
                if (old != null){
                    old.scale(Entity3d.SCALE.SHRINK);
                }
            }

            // Change the monitored entity.
            this.entity = entity;
            // Notify it that it is being monitored.
            entity.setMonitored(true);
            Entity3d e = entity.getEntity3d();

            if (e != null){
                e.scale(Entity3d.SCALE.ENLARGE);
            }
            
        } else {
            this.entity = null;
        }

        // There is something new to monitor!
        // Wake up the sleeping run method.
        synchronized (lock){
            lock.notifyAll();
        }
    }

    /**
     * Updates the gui fields with new information about the monitored entity.
     */
    private synchronized void update() {

        if (entity == null){
            // clear all fields
            setId("");
            position.setText("");
            capabilities.setText("");
            return;
        }

        // If the monitored entity is a Node,
        if (entity.isNode()){
            Node tmp = (Node) entity;
            // If the user wants to monitor a different Node,
            if (entityChanged){
                setId(tmp.getId());
                Vector3d v =  tmp.getPosition().getVector3d();
                setPosition(v.getX(), v.getY(), v.getZ());
                entityChanged = false;
            }

        // If it is a Link,
        } else {
            Link tmp = (Link) entity;
            // If the user wants to monitor a different Link,
            if (entityChanged){
                setId(tmp.getSource().getId() + "\n" + tmp.getTarget().getId());
                position.setText("");
                entityChanged = false;
            }
        }

        capabilities.setText("");
        synchronized (entity.lock){
            for (Capability cap : entity.getCapabilitySet()){
                addCapability(cap.getName(), cap.getValue());
            }
        }
    }

    
    private void setId(String id){
        nodeId.setText(id);
    }

    private void setPosition(double x, double y, double z){
        position.setText("  x = " + x + "\n  y = " + y + "\n  z = " + z);
    }

    private void addCapability(String name, String value){
        capabilities.append("\n\n" + name + ": " + value);
    }

    public void setTime(double time) {
        this.time.setText(format.format(new Date(Setup.getStart().getTime() + (long)time)));
    }
}
