package visual;

import walker.utilities.Rendering;

import javax.swing.*;

/**
 * Holds the jdialog used to indicate the seeking progress.
 */
public class Message extends JDialog {
    private JPanel contentPane;
    private JTextArea textArea1;
    private JProgressBar seekingProgress;
    private double seeked;
    private double timespan;

    /**
     * Resets the seek bar and displays the dialog.
     *
     * @param text to show.
     * @param seeked point in experiment time where the user want's to seek to.
     */
    public void show(String text, double seeked){
        textArea1.setText(text);
        this.seeked = seeked;
        timespan = -1;
        setVisible(true);
        setLocation(Rendering.getScreenHeight()/2, Rendering.getScreenHeight()/2);
        setAlwaysOnTop(true);
        pack();
    }

    /**
     * Default Constructor.
     */
    public Message() {
        setContentPane(contentPane);
    }

    /**
     * Updates the seeking progress.
     *
     * @param currentTime the current point in time.
     */
    public void setProgress(double currentTime){

        if (timespan == -1){
            timespan = seeked - currentTime;
        }
        
        seekingProgress.setValue((int)((timespan - (seeked-currentTime))*100/timespan));
    }

    /**
     * Disposes the window and cleans all data.
     */
    public void destroy(){
        timespan = -1;
        seeked = -1;
        seekingProgress.setValue(0);
        dispose();
    }

}
